#!/usr/bin/env python
import datetime
from subprocess import call
import sys
import traceback

class TestError(StandardError):
    pass

try:
    # Modul teszt:
    try:
        import esemeny
    except ImportError:
        raise TestError('HIBA: Nincs meg esemeny.py nevu fajl a konyvtarban!')

    print 'SIKER: esemeny nevu modul letezik es imporalhato.'

    # Osztaly alap tesztek:
    try:
        if type(esemeny.Raeres).__name__ != 'type':
            raise TestError('HIBA: A Raeres nevu osztaly rosszul van definialva. (Lehet hogy kimaradt az "object"?)')
    except AttributeError:
        print
        traceback.print_exc(2, sys.stdout)
        raise TestError('HIBA: Nincs Raeres nevu osztaly a modulban!')

    t1_nev = 'Solas'
    t1_datumok = [datetime.date(2015, 3, 5)]
    try:
        t1 = esemeny.Raeres(t1_nev, t1_datumok)
    except TypeError:
        print
        traceback.print_exc(2, sys.stdout)
        raise TestError('HIBA: A Raeres osztaly konstruktor nem ket parametert var (a self-en kivul)')

    print 'SIKER: A Raeres osztaly letezik, ket parametert varo konstruktorral.'
    t1_dict = t1.__dict__
    try:
        if t1_dict['nev'] != t1_nev:
            raise TestError('HIBA: A Raeres osztaly kontruktora nem megfeleloen tarolja le a nevet.')
    except KeyError:
        raise TestError('HIBA: A Raeres osztaly kontruktora nem csinal "nev" nevu tagvaltozot!')

    try:
        if t1_dict['datumok'] != t1_datumok:
            raise TestError('HIBA: A Raeres osztaly kontruktora nem megfeleloen tarolja le a datumokat.')
    except KeyError:
        raise TestError('HIBA: A Raeres osztaly kontruktora nem csinal "datumok" nevu tagvaltozot!')

    # Metodusok tesztjei:

    # 1: konstruktor
    try:
        t2_nev = "Varric"
        t2_datumok = [datetime.date(2016, 2, 2), datetime.date(2016, 1, 2)]
        t2_datumok_sorban = [datetime.date(2016, 1, 2), datetime.date(2016, 2, 2)]

        t2 = esemeny.Raeres(t2_nev, t2_datumok)
        if t2.datumok != t2_datumok_sorban:
            raise TestError('HIBA: A Raeres osztaly kontruktora nem rendezi sorba a datumokat!')

        print "SIKER: A Raeres osztaly konstruktora jol mukodik"

        # 2: megis()
        try:
            try:
                t2.megis(datetime.date(2016, 1, 20))
                t2_megis_utan = [datetime.date(2016, 1, 2), datetime.date(2016, 1, 20), datetime.date(2016, 2, 2)]
                if t2.datumok != t2_megis_utan:
                    raise TestError('HIBA: A Raeres osztaly megis() metodusa rossz eredmenyt ad. A datumok tagvaltozo\n{}\nlett\n{}\nhelyett'.format(t2.datumok, t2_megis_utan))
                print "SIKER: A Raeres osztaly megis() metodusa jol mukodik."

            except AttributeError:
                print
                traceback.print_exc(2, sys.stdout)
                raise TestError('HIBA: A Raeres osztalynak nincs megis() nevu metodusa. (Vagy esetleg a metodus rosszul hasznal valamit.)')
        except TestError as e:
            print e.message

        # 2: megse()
        try:
            try:
                t3 = esemeny.Raeres("Cassandra", [datetime.date(2014, 5, 2), datetime.date(2015, 1, 19), datetime.date(2014, 12, 25), datetime.date(2014, 9, 12), datetime.date(2014, 9, 13)])
                t3.megse(datetime.date(2014, 12, 25))
                t3_megse_utan = [datetime.date(2014, 5, 2), datetime.date(2014, 9, 12), datetime.date(2014, 9, 13), datetime.date(2015, 1, 19)]
                if t3.datumok != t3_megse_utan:
                    raise TestError('HIBA: A Raeres osztaly megse() metodusa rossz eredmenyt ad. A datumok tagvaltozo\n{}\nlett\n{}\nhelyett'.format(t3.datumok, t3_megse_utan))
                print "SIKER: A Raeres osztaly megse() metodusa jol mukodik."

            except AttributeError:
                print
                traceback.print_exc(2, sys.stdout)
                raise TestError('HIBA: A Raeres osztalynak nincs megse() nevu metodusa. (Vagy esetleg a metodus rosszul hasznal valamit.)')
        except TestError as e:
            print e.message


    except TestError as e:
        print e.message

    # legjobb_datum tesztje:
    hibas_teszt_str = "HIBA: esemeny.legjobb_datum() rossz eredmenyt ad. Parameter raeresek:\n{}\nElvart visszateresi ertek:\n{}\nKapott kimenet:\n{}"
    try:
        try:
            leg_t1_in = "[esemeny.Raeres('Solas', [datetime.date(2015, 3, 5)])]"
            leg_t1_out = "datetime.date(2015, 3, 5)"
            leg_t1_ret = esemeny.legjobb_datum(eval(leg_t1_in))

        except AttributeError:
            print
            traceback.print_exc(2, sys.stdout)
            raise TestError('HIBA: nincs a modulban legjobb_datum() nevu fuggveny. (Vagy esetleg a fuggveny rosszul hasznal valamit.)')

        # t1
        try:
            if leg_t1_ret != eval(leg_t1_out):
                raise TestError(hibas_teszt_str.format(leg_t1_in, leg_t1_out, repr(leg_t1_ret)))
            print "SIKER: esemeny.legjobb_datum() 1. teszt"
        except TestError as e:
            print e.message

        # t2
        try:
            leg_t2_in = "[esemeny.Raeres('Blackwall', [datetime.date(2015, 3, 5), datetime.date(2015, 3, 6)])]"
            leg_t2_out = "datetime.date(2015, 3, 5)"
            leg_t2_ret = esemeny.legjobb_datum(eval(leg_t2_in))
            if leg_t2_ret != eval(leg_t2_out):
                raise TestError(hibas_teszt_str.format(leg_t2_in, leg_t2_out, repr(leg_t2_ret)))
            print "SIKER: esemeny.legjobb_datum() 2. teszt"
        except TestError as e:
            print e.message

        # t3
        try:
            leg_t3_in = "[esemeny.Raeres('Blackwall', [datetime.date(2015, 3, 6), datetime.date(2015, 3, 5)])]"
            leg_t3_out = "datetime.date(2015, 3, 5)"
            leg_t3_ret = esemeny.legjobb_datum(eval(leg_t3_in))
            if leg_t3_ret != eval(leg_t3_out):
                raise TestError(hibas_teszt_str.format(leg_t3_in, leg_t3_out, repr(leg_t3_ret)))
            print "SIKER: esemeny.legjobb_datum() 3. teszt"
        except TestError as e:
            print e.message

        # t4
        try:
            leg_t4_in = """[esemeny.Raeres('Solas', [datetime.date(2015, 3, 5), datetime.date(2015, 3, 6)]),
 esemeny.Raeres('Varric', [datetime.date(2015, 3, 7), datetime.date(2015, 3, 6)])]"""
            leg_t4_out = "datetime.date(2015, 3, 6)"
            leg_t4_ret = esemeny.legjobb_datum(eval(leg_t4_in))
            if leg_t4_ret != eval(leg_t4_out):
                raise TestError(hibas_teszt_str.format(leg_t4_in, leg_t4_out, repr(leg_t4_ret)))
            print "SIKER: esemeny.legjobb_datum() 4. teszt"
        except TestError as e:
            print e.message

        # t5
        try:
            leg_t5_in = "[esemeny.Raeres('Cassandra', [datetime.date(2015, 3, 27), datetime.date(2015, 3, 28)])]"
            leg_t5_out = "datetime.date(2015, 3, 28)"
            leg_t5_ret = esemeny.legjobb_datum(eval(leg_t5_in))
            if leg_t5_ret != eval(leg_t5_out):
                raise TestError(hibas_teszt_str.format(leg_t5_in, leg_t5_out, repr(leg_t5_ret)))
            print "SIKER: esemeny.legjobb_datum() 5. teszt"
        except TestError as e:
            print e.message

        # t6
        try:
            leg_t6_in = """[esemeny.Raeres('Cassandra', [datetime.date(2015, 3, 24), datetime.date(2015, 3, 23)]),
 esemeny.Raeres('Dorian', [datetime.date(2015, 3, 25), datetime.date(2015, 3, 24)]),
 esemeny.Raeres('Cole', [datetime.date(2015, 3, 25), datetime.date(2015, 3, 23)])]"""
            leg_t6_out = "datetime.date(2015, 3, 23)"
            leg_t6_ret = esemeny.legjobb_datum(eval(leg_t6_in))
            if leg_t6_ret != eval(leg_t6_out):
                raise TestError(hibas_teszt_str.format(leg_t6_in, leg_t6_out, repr(leg_t6_ret)))
            print "SIKER: esemeny.legjobb_datum() 6. teszt"
        except TestError as e:
            print e.message

        # t7
        try:
            leg_t7_in = """[esemeny.Raeres('Cassandra', [datetime.date(2015, 3, 24), datetime.date(2015, 3, 23)]),
 esemeny.Raeres('Dorian', [datetime.date(2015, 3, 25), datetime.date(2015, 3, 24)]),
 esemeny.Raeres('Cole', [datetime.date(2015, 3, 25), datetime.date(2015, 3, 24), datetime.date(2015, 3, 23)])]"""
            leg_t7_out = "datetime.date(2015, 3, 24)"
            leg_t7_ret = esemeny.legjobb_datum(eval(leg_t7_in))
            if leg_t7_ret != eval(leg_t7_out):
                raise TestError(hibas_teszt_str.format(leg_t7_in, leg_t7_out, repr(leg_t7_ret)))
            print "SIKER: esemeny.legjobb_datum() 7. teszt"
        except TestError as e:
            print e.message

        # t8
        try:
            leg_t8_in = """[esemeny.Raeres('Cassandra', [datetime.date(2015, 3, 24), datetime.date(2015, 3, 23)]),
 esemeny.Raeres('Dorian', [datetime.date(2015, 3, 25), datetime.date(2015, 3, 24)]),
 esemeny.Raeres('Cole', [datetime.date(2015, 3, 25), datetime.date(2015, 3, 24), datetime.date(2015, 3, 23)])]"""
            leg_t8_out = "datetime.date(2015, 3, 24)"
            leg_t8_ret = esemeny.legjobb_datum(eval(leg_t8_in))
            if leg_t8_ret != eval(leg_t8_out):
                raise TestError(hibas_teszt_str.format(leg_t8_in, leg_t8_out, repr(leg_t8_ret)))
            print "SIKER: esemeny.legjobb_datum() 8. teszt"
        except TestError as e:
            print e.message

        # t9
        try:
            leg_t9_in = """[esemeny.Raeres('Cassandra', [datetime.date(2015, 3, 24), datetime.date(2015, 3, 23)]),
 esemeny.Raeres('Dorian', [datetime.date(2015, 3, 25), datetime.date(2015, 3, 24)]),
 esemeny.Raeres('Cole', [datetime.date(2015, 3, 25), datetime.date(2015, 3, 24), datetime.date(2015, 3, 23)]),
 esemeny.Raeres('Cullen', [datetime.date(2015, 3, 25), datetime.date(2015, 3, 23)])]"""
            leg_t9_out = "datetime.date(2015, 3, 23)"
            leg_t9_ret = esemeny.legjobb_datum(eval(leg_t9_in))
            if leg_t9_ret != eval(leg_t9_out):
                raise TestError(hibas_teszt_str.format(leg_t9_in, leg_t9_out, repr(leg_t9_ret)))
            print "SIKER: esemeny.legjobb_datum() 9. teszt"
        except TestError as e:
            print e.message

        # t10
        try:
            leg_t10_in = """[esemeny.Raeres('Cassandra', [datetime.date(2015, 3, 24), datetime.date(2015, 3, 28), datetime.date(2015, 3, 23)]),
 esemeny.Raeres('Dorian', [datetime.date(2015, 3, 25), datetime.date(2015, 3, 28)]),
 esemeny.Raeres('Cole', [datetime.date(2015, 3, 25), datetime.date(2015, 3, 24), datetime.date(2015, 3, 24), datetime.date(2015, 3, 23)]),
 esemeny.Raeres('Cullen', [datetime.date(2015, 3, 25), datetime.date(2015, 3, 28), datetime.date(2015, 3, 23)])]"""
            leg_t10_out = "datetime.date(2015, 3, 28)"
            leg_t10_ret = esemeny.legjobb_datum(eval(leg_t10_in))
            if leg_t10_ret != eval(leg_t10_out):
                raise TestError(hibas_teszt_str.format(leg_t10_in, leg_t10_out, repr(leg_t10_ret)))
            print "SIKER: esemeny.legjobb_datum() 10. teszt"
        except TestError as e:
            print e.message

        # t11
        try:
            leg_t11_in = """[esemeny.Raeres('Cassandra', [datetime.date(2015, 3, 29), datetime.date(2015, 3, 28)]),
 esemeny.Raeres('Dorian', [datetime.date(2015, 3, 30), datetime.date(2015, 3, 29)]),
 esemeny.Raeres('Cole', [datetime.date(2015, 3, 30), datetime.date(2015, 3, 29), datetime.date(2015, 3, 28)]),
 esemeny.Raeres('Cullen', [datetime.date(2015, 3, 30), datetime.date(2015, 3, 28)])]"""
            leg_t11_out = "datetime.date(2015, 3, 28)"
            leg_t11_ret = esemeny.legjobb_datum(eval(leg_t11_in))
            if leg_t11_ret != eval(leg_t11_out):
                raise TestError(hibas_teszt_str.format(leg_t11_in, leg_t11_out, repr(leg_t11_ret)))
            print "SIKER: esemeny.legjobb_datum() 11. teszt"
        except TestError as e:
            print e.message

    except TestError as e:
        print e.message

    # legjobb_fajlbol tesztje:
    hibas_teszt_str = "HIBA: esemeny.legjobb_fajlbol() rossz eredmenyt ad. Parameter raeresek:\n{}\nElvart visszateresi ertek:\n{}\nKapott kimenet:\n{}"
    try:
        try:
            leg_t1_in = "t1.txt"
            leg_t1_out = "datetime.date(2015, 3, 5)"
            leg_t1_ret = esemeny.legjobb_fajlbol(leg_t1_in)

        except AttributeError:
            print
            traceback.print_exc(2, sys.stdout)
            raise TestError('HIBA: nincs a modulban legjobb_fajlbol() nevu fuggveny. (Vagy esetleg a fuggveny rosszul hasznal valamit.)')

        # t1
        try:
            if leg_t1_ret != eval(leg_t1_out):
                raise TestError(hibas_teszt_str.format(leg_t1_in, leg_t1_out, repr(leg_t1_ret)))
            print "SIKER: esemeny.legjobb_fajlbol() 1. teszt"
        except TestError as e:
            print e.message

        # t2
        try:
            leg_t2_in = "t2.txt"
            leg_t2_out = "datetime.date(2015, 3, 3)"
            leg_t2_ret = esemeny.legjobb_fajlbol(leg_t2_in)
            if leg_t2_ret != eval(leg_t2_out):
                raise TestError(hibas_teszt_str.format(leg_t2_in, leg_t2_out, repr(leg_t2_ret)))
            print "SIKER: esemeny.legjobb_fajlbol() 2. teszt"
        except TestError as e:
            print e.message

        # t3
        try:
            leg_t3_in = "t3.txt"
            leg_t3_out = "datetime.date(2015, 3, 28)"
            leg_t3_ret = esemeny.legjobb_fajlbol(leg_t3_in)
            if leg_t3_ret != eval(leg_t3_out):
                raise TestError(hibas_teszt_str.format(leg_t3_in, leg_t3_out, repr(leg_t3_ret)))
            print "SIKER: esemeny.legjobb_fajlbol() 3. teszt"
        except TestError as e:
            print e.message

    except TestError as e:
        print e.message

    try:
        with open("legjobb_datum.txt", "w") as fajl:
            fajl.write("\n")
        call(["python", "esemeny.py"])
        with open("legjobb_datum.txt", "r") as fajl:
            if fajl.read().strip() != "2015-03-28":
                raise TestError('HIBA: Rossz kimenet a legjobb_datum.txt-ben. Elvart kimenet egy sor, aminek tartalma:\n2015-03-28')
        print "SIKER: esemeny.py fajl magaban futtatasa"
    except OSError:
        print
        traceback.print_exc(2, sys.stdout)
        raise TestError('HIBA: nem sikerult kulon futtatni a pythont, lehet hogy mashogy van telepitve a gepedre. Ha minden mas teszt jo, es kezzel kiprobalva mukodik az esemeny.py fajl kulon futtatasa, akkor ezt a hibat nem kell figyelembe venni.')
    except IOError:
        print
        traceback.print_exc(2, sys.stdout)
        raise TestError('HIBA: esemeny.py fajl kulon futtatasa nem hozott letre legjobb_datum.txt nevu fajlt.')




except TestError as e:
    print e.message

raw_input("Enter-el tudsz kilepni!")
